(* ::Package:: *)

c=2.998*10^8 \[ThickSpace]; (* Speed of light *)
e=1.6021766*10^-19; (* Electron's charge*)
\[HBar]=1.054572*10^-34; 
a0=5.29177 * 10^-11; (* Bohr radius *)
n=1; (* Index of refraction of air *)
\[Epsilon]0=8.854*10^-12; (* vacuum permittivity *)
kBoltz=1.38064852*10^-23; (* Boltzmann constant *)


BareGlassCell ={
	WallBounces -> 0, (* no coating *)
	Acell -> 0.01^2, (* Cell area in m^2 *)
	l->0.01(* Cell length in m *)
} // N;


ParaffinCell ={
	WallBounces -> 10^4, (* Bouchiat and Brossel *)
	Acell -> 0.01^2, (* Cell area in m^2 *)
	l->0.01(* Cell length in m *)
} // N;


BufferGasData = Module[
	{\[Sigma]SDmap, diffusion, broadening, shift}, 
	(* what about spin-exchange rates? *)

	(* ignore for now *)
	shift = <||>;

	(* Spin-destruction cross-sections table in m^2 *)
	\[Sigma]SDmap = <|
		"Rb" -> <|
			"N2" -> 1*10^-22/10^4, (* Seltzer thesis *)
			"He" -> 9*10^-24/10^4, (* Seltzer thesis *)
			"Xe" -> 2*10^-19/10^4 (* Seltzer thesis *)
		|>,
		"Cs" -> <|
			"N2" -> 5.5*10^-22/10^4, (* Seltzer thesis *)
			"He" -> 2.8*10^-23/10^4, (* Seltzer thesis *)
			"Xe" -> 2.3*10^-19/10^4 (* Seltzer thesis *)
		|>
	|>;

	(* Table of diffusion constants [m^2/s] at 1 amg at 273 K  *)
	diffusion = <|
		"Rb" -> <|
			"He" -> 0.5 /10^4, (* Seltzer thesis *)
			"N2" -> 0.19 /10^4 (* Seltzer thesis *)
		|>,
		"Cs" -> <|
			"He" -> 0.29 /10^4, (* Seltzer thesis *)
			"N2" -> 0.1 /10^4 (* Seltzer thesis *)
		|>
	|>;

	(* Table of line broadening coefficients  Hz/amg, FWHM at 273 K  *)
	broadening = <|
		"Rb" -> <|
			"D1" -> <|
				"He" -> 18 * 10^9, (* Seltzer thesis *)
				"N2" -> 17.8 * 10^9 (* Seltzer thesis *)
			|>,
			"D2" -> <|
				"He" -> 15.8 * 10^9, (* Seltzer thesis *)
				"N2" -> 18.1 * 10^9 (* Seltzer thesis *)
			|>
		|>,
		"Cs" -> <|
			"D1" -> <|
				"He" -> 19.92 * 10^9, (* Seltzer thesis *)
				"N2" -> 14.83 * 10^9 (* Seltzer thesis *)
			|>,
			"D2" -> <|
				"He" -> 17.86 * 10^9, (* Seltzer thesis *)
				"N2" -> 17.24 * 10^9 (* Seltzer thesis *)
			|>
		|>
	|>;

	<| "\[Sigma]SD" -> \[Sigma]SDmap, "D0" -> diffusion, "\[CapitalGamma]" -> broadening |> // N
];


Cs133 = {
	element -> "Cs",
	NuclearSpin -> 7/2,
(* Mass of an atom in SI units *)
	AtomMass -> 133 * 1.66054*10^-27,

(* Fit constants for the vapor density of alkali atoms, from S. Seltzer dissertation  *)
	VaporDensityA-> If[Temperature < 28.5+273.15, 4.711, 4.165],
	VaporDensityB->If[Temperature < 28.5+273.15,3999, 3830],

(* Spin-exchange collision cross-section in m^2 *)
	\[Sigma]SE -> 2.1*10^-14 / 10^4, 

(* Spin-destruction collision cross-section in m^2 *)
	\[Sigma]SD -> 2.0*10^-16/ 10^4,

(* gyromagnetic ratio for the cases when nuclear and atomic moments are parallel
 and anti-parallel in Hz/T *)
	\[Gamma]par->3.498 * 10^9,
	\[Gamma]antipar->-3.509 * 10^9
} ;


AmagatFromTorr[x_] := x / 760 (* 1 Amagat = 760 Torr at 0 deg C *)


Rb87 = {
	element -> "Rb",
	NuclearSpin -> 3/2,
(* Mass of an atom in SI units *)
	AtomMass -> 87 * 1.66054*10^-27,

(* Fit constants for the vapor density of alkali atoms, from S. Seltzer dissertation  *)
	VaporDensityA-> If[Temperature < 39.3+273.15, 4.857, 4.312],
	VaporDensityB->If[Temperature < 39.3+273.15,4215, 4040],

(* Spin-exchange collision cross-section in m^2 *)
	\[Sigma]SE -> 1.9*10^-14 / 10^4, 

(* Spin-destruction collision cross-section in m^2 *)
	\[Sigma]SD -> 1.6*10^-16/ 10^4,

(* gyromagnetic ratio for the cases when nuclear and atomic moments are parallel
 and anti-parallel in Hz/T *)
	\[Gamma]par->6.995 * 10^9, 
	\[Gamma]antipar->-7.022 * 10^9
};


Rb87D2 = {
	line -> "D2",
(* Resonant absorption cross-section for the cycling transition *)
	\[Sigma]0plus->2.9*10^-9*10^-4 ,

(* Unpolarized absorption cross-section *)
	\[Sigma]0minus->1.3*10^-9*10^-4,

(* Unpolarized absorption cross-section *)
	\[Sigma]0unpol->1.3*10^-9*10^-4,

(* Average dipole element squared *)
	DipoleMomentSquared -> 1.18 (e a0)^2, 

(* Natural decay rate Rb D2 *)
	\[CapitalGamma]0 -> 38.116 * 10^6, 

(* Optical transition frequency *)
	f0 -> 384.230 * 10^12, 

(* How many photons on average does it take to bring the atom into a stretched state if there were no relaxation*)
	AvgPhotonsToStretchedState -> 2
} ;


Cs133D2 = {
	line -> "D2",
(* Resonant absorption cross-section for the cycling transition *)
	\[Sigma]0plus->3.5*10^-9*10^-4 ,

(* Unpolarized absorption cross-section *)
	\[Sigma]0minus->1.1*10^-9*10^-4 ,

(* Unpolarized absorption cross-section *)
	\[Sigma]0unpol->1.1*10^-9*10^-4 ,

(* Average dipole element squared *)
	DipoleMomentSquared -> 1.18 (e a0)^2, 

(* Natural decay rate Cs D2 *)
	\[CapitalGamma]0 -> 32.815 * 10^6, 

(* Optical transition frequency *)
	f0 -> 351.725 * 10^12, 

(* How many photons on average does it take to bring the atom into a stretched state if there were no relaxation*)
	AvgPhotonsToStretchedState -> 4 
};


SlowingDownFactor[\[Alpha]_, NuclearSpin_]:=Which[
		(* From Selzer thesis *)
		NuclearSpin == 3/2, (6 + 2\[Alpha]^2)/(1 + \[Alpha]^2),
		NuclearSpin == 5/2, (38 + 52\[Alpha]^2 + 6\[Alpha]^4)/(3 + 10\[Alpha]^2 + 3\[Alpha]^4),
		NuclearSpin == 7/2, (22 + 70\[Alpha]^2 + 34\[Alpha]^4 + 2\[Alpha]^6)/(1 + 7\[Alpha]^2 + 7\[Alpha]^4 + \[Alpha]^6)
	]


AvgSpeed[atoms_,cell_]:=Sqrt[8 * kBoltz *  Temperature / \[Pi]/ AtomMass]/. cell /. atoms


WallToWallDistance[cell_]:= Min[l, Sqrt[Acell]] /. cell


T1WallLimited[atoms_, cell_, gases_:{}]:=Module[{RmsSpeed, T1WallLimited},
	RmsSpeed=Sqrt[3 * kBoltz *  Temperature / AtomMass]/. cell /. atoms;
	T1WallLimited=(WallToWallDistance[cell]*WallBounces/AvgSpeed[atoms, cell]) /. cell;
	T1WallLimited
]


natoms[atoms_, cell_]:=1/Temperature * 10^(21.866+VaporDensityA-VaporDensityB/Temperature) * 10^6 /. atoms /. cell


T1SDLimited[atoms_, cell_]:=1/(natoms[atoms,cell]*\[Sigma]SD*Sqrt[2]*AvgSpeed[atoms,cell]) /. atoms 


T1[atoms_, cell_] := 1/(1/T1SDLimited[atoms, cell] + 1/T1WallLimited[atoms, cell])


T2SpinExchange[atoms_, cell_] := 1/(natoms[atoms, cell]*\[Sigma]SE*Sqrt[2]*AvgSpeed[atoms, cell]) /. atoms 


T2[atoms_, cell_] := 1/(1/T1[atoms, cell] + 1/T2SpinExchange[atoms, cell])


MagneticResonanceFWHM[atoms_,cell_] := 1/\[Pi]/T2[atoms,cell]


DopplerHWHM[atoms_, cell_, transition_] := AvgSpeed[atoms, cell]/c * f0 /.transition 


BufferGasOpticalFWHM[{gas_,torr_},atoms_, line_]:=BufferGasData["\[CapitalGamma]"][atoms][line][gas] * AmagatFromTorr[torr] // N


BroadenedOpticalFWHM[atoms_,cell_,transition_, gases_:{}]:= Module[{DopplerFWHM, BufferFWHM},
	(* 
		Gases are in format {{"gas1", torr1}, {"gas2", torr2}, ... } 
	*)
	DopplerFWHM = 2 * DopplerHWHM[atoms,cell,transition];
	BufferFWHM = BufferGasOpticalFWHM[#, element /. atoms , line /. transition]&/@ gases;
	Sqrt[Total[BufferFWHM^2] + DopplerFWHM^2]
]


DopplerFactor[atoms_, cell_, transition_] := Module[
	{\[CapitalGamma]DopplerBroadened, DopplerFactor},
	\[CapitalGamma]DopplerBroadened = 2*AvgSpeed[atoms, cell]/c* f0 /.transition ;
	DopplerFactor=\[CapitalGamma]0/\[CapitalGamma]DopplerBroadened  /. transition ;
	DopplerFactor
]


Doppler[atoms_,cell_,transition_,A_,\[CapitalDelta]f_]:=A *  Exp[-AtomMass * (\[CapitalDelta]f / f0 * c)^2 / (2 kBoltz Temperature)] /.atoms /.cell /.transition


DispersiveLorentz[atoms_,cell_,transition_,A_,\[CapitalDelta]f_]:=Module[{\[CapitalGamma]},
	\[CapitalGamma] = DopplerHWHM[atoms, cell, transition];
	2 * A * \[CapitalGamma] * \[CapitalDelta]f / ( \[CapitalGamma]^2 + \[CapitalDelta]f^2)  /.atoms /.transition
]


CellPolarizationDegree[atoms_,cell_,lasers_,transition_]:=Module[
	{MagneticResonanceAmplitude, Elight, \[CapitalOmega]r, \[CapitalGamma]pmp, \[CapitalGamma]pmpDopp, l0plus, l0minus, \[CapitalGamma]rel, \[Alpha], \[CapitalDelta]\[Phi],
		\[CapitalDelta]\[Phi]degDetunedEstimate, \[CapitalDelta]\[Phi]dopp, CharacteristicLength, AbsorptionLength},
	(* formulas refer to the optically pumped atoms book by Dima *)
	(* Magnetic reconance amplitude is in radians *)

	Elight=Sqrt[2 PumpPower/Acell/c/n/\[Epsilon]0] /. lasers /. cell; (* 1.5 *)
	\[CapitalOmega]r=d Elight/\[HBar]; (* 1.3 *)
	\[CapitalGamma]pmp = \[CapitalOmega]r^2/\[CapitalGamma]0/. d^2->DipoleMomentSquared /.transition; (* 1.2 *)
	\[CapitalGamma]pmpDopp=\[CapitalGamma]pmp*DopplerFactor[atoms, cell, transition];

	\[CapitalGamma]rel=1/T2[atoms, cell] ;
	CharacteristicLength = WallToWallDistance[cell];
	AbsorptionLength = 1/natoms[atoms, cell]/\[Sigma]0unpol /. atoms;

	(* 
		p - polarized
		u - unpolarized

		processes taken into account (left to right):
			- optical pumping
			- spontaneous decay
			- optical trapping
	*)

	\[Alpha] = p /.Solve[
	{
		(* dp/dt = *) 0 == u * (\[CapitalGamma]pmpDopp/AvgPhotonsToStretchedState) - p * \[CapitalGamma]rel - p * CharacteristicLength / 2 / AbsorptionLength, 
		(* du/dt = *)0 == -u * (\[CapitalGamma]pmpDopp/AvgPhotonsToStretchedState) + p * \[CapitalGamma]rel + p * CharacteristicLength / 2 / AbsorptionLength,
		p + u == 1
	} /. transition, {u, p}][[1]]
]


MagneticResonanceAmplitude[atoms_,cell_,lasers_,transition_]:=Module[
	{MagneticResonanceAmplitude, l0plus, l0minus, \[CapitalGamma]rel, \[Alpha], \[CapitalDelta]\[Phi],
		\[CapitalDelta]\[Phi]degDetunedEstimate, \[CapitalDelta]\[Phi]dopp},
	(* formulas refer to the optically pumped atoms book by Dima *)
	(* Magnetic reconance amplitude is in radians *)
	
	l0plus=1/natoms[atoms, cell]/\[Sigma]0plus /. atoms;
	l0minus=1/natoms[atoms, cell]/\[Sigma]0minus /. atoms; 

	\[Alpha] = CellPolarizationDegree[atoms, cell, lasers, transition];
	\[CapitalDelta]\[Phi] = \[Alpha] l /5  *(1/l0plus - 1/l0minus) /. cell /. transition /. atoms; (* 1.14 *)
	\[CapitalDelta]\[Phi]dopp = \[CapitalDelta]\[Phi] * DopplerFactor[atoms, cell, transition];
	\[CapitalDelta]\[Phi]degDetunedEstimate = \[CapitalDelta]\[Phi]dopp*180/\[Pi]*ProbeDetuningReductionCoef /. lasers;
	MagneticResonanceAmplitude = \[CapitalDelta]\[Phi]dopp*ProbeDetuningReductionCoef /. lasers;
	MagneticResonanceAmplitude
]


PhotonShotNoiseFtSqrtHz[atoms_,cell_,lasers_,transition_]:=Module[
	{\[CapitalPhi]photon, PhotonShotNoise, SignalToNoise, PhotonShotNoiseFtSqrtHz},

(* Photon flux in photons per second *)
	\[CapitalPhi]photon = ProbePower / (\[HBar] * 2 \[Pi] * f0 ) /. lasers /.transition;

(* Photon shot noise in radians / sqrt[Hz] *)
	PhotonShotNoise = 1/2/Sqrt[\[CapitalPhi]photon] ;

	PhotonShotNoiseFtSqrtHz = 2 * MagneticResonanceFWHM[atoms, cell]  (* Hz *)* 
		(PhotonShotNoise (* radians*)/ MagneticResonanceAmplitude[atoms, cell, lasers, transition] (* radians*))
			/ \[Gamma]par (* Hz/T *) * 10^15 /. atoms;
	PhotonShotNoiseFtSqrtHz
]


SpinProjectionNoiseFtSqrtHz[atoms_, cell_, lasers_, transition_] := Module[{\[CapitalGamma]rel},
	\[CapitalGamma]rel=1/T2[atoms, cell];
	1/\[Gamma]par * Sqrt[\[CapitalGamma]rel/(CellPolarizationDegree[atoms, cell, lasers, transition] 
		* natoms[atoms, cell])/1]*10^15/.atoms 
]


TotalNoise[atoms_,cell_,lasers_,transition_] := 
	Sqrt[
		SpinProjectionNoiseFtSqrtHz[atoms, cell, lasers,transition]^2 
		+ PhotonShotNoiseFtSqrtHz[atoms, cell, lasers,transition]^2
	] 


RotationPerFtRadians[atoms_,cell_,lasers_,transition_]:= 10^-15 (* T *) / 2 * MagneticResonanceAmplitude[atoms, cell, lasers, transition] (* radians*) * 
	\[Gamma]par (* Hz / T*) / MagneticResonanceFWHM[atoms, cell]  (* Hz *)  /. atoms
