(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     59534,       1163]
NotebookOptionsPosition[     58263,       1112]
NotebookOutlinePosition[     58608,       1127]
CellTagsIndexPosition[     58565,       1124]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Synchronously pumped magnetometer with coated cells, performance vs \
temperature\
\>", "Title",
 CellChangeTimes->{{3.6803915009545293`*^9, 3.6803915280930085`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lasers", " ", "=", " ", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PumpPower", " ", "->", " ", 
       RowBox[{"30", "\[Times]", 
        SuperscriptBox["10", 
         RowBox[{"-", "6"}]]}]}], ",", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"10", " ", "microwatts"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"ProbePower", " ", "->", " ", 
       RowBox[{"10", "\[Times]", 
        SuperscriptBox["10", 
         RowBox[{"-", "6"}]]}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"ProbeDetuningReductionCoef", " ", "\[Rule]", "0.1"}]}], 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "reduction", " ", "in", " ", "optical", " ", "rotation", " ", "due", 
       " ", "to", " ", "the", " ", "probe", " ", "being", " ", "detuned", " ",
        "from", " ", "transition", " ", 
       RowBox[{"(", 
        RowBox[{
        "for", " ", "example", " ", "to", " ", "avoid", " ", "the", " ", "LS",
          " ", "noise", " ", "or", " ", "depopulating", " ", "the", " ", 
         "state", " ", "by", " ", "the", " ", "probe"}], ")"}]}], " ", "*)"}],
      "\[IndentingNewLine]", "}"}], " ", "//", " ", "N"}]}], " ", 
  ";"}]], "Input",
 CellChangeTimes->{
  3.6803905992239933`*^9, {3.6803922270402236`*^9, 3.6803922396405344`*^9}, {
   3.6803923692500057`*^9, 3.6803923855433965`*^9}, {3.680537460667796*^9, 
   3.6805374607324066`*^9}, {3.6805379574974737`*^9, 3.68053795801311*^9}, 
   3.6805379887906466`*^9}],

Cell[CellGroupData[{

Cell["T1 and T2 for Cs133", "Section",
 CellChangeTimes->{{3.680391535242239*^9, 3.6803915524828544`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"T1", "[", 
       RowBox[{"Cs133", ",", " ", "ParaffinCell"}], "]"}], ",", " ", 
      "\[IndentingNewLine]", 
      RowBox[{"T2", "[", 
       RowBox[{"Cs133", ",", " ", "ParaffinCell"}], "]"}]}], "}"}], " ", "/.",
     " ", 
    RowBox[{"Temperature", "\[Rule]", 
     RowBox[{"t", "+", "273"}]}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", " ", "0", ",", " ", "60"}], "}"}], ",", " ", 
   RowBox[{"Evaluated", "\[Rule]", "True"}], ",", " ", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Temperature, C\>\"", ",", " ", "\"\<s\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.680387646620475*^9, 3.6803876789640713`*^9}, {
   3.6803880754049187`*^9, 3.68038817015278*^9}, 3.6803882253739305`*^9, {
   3.6803882751463428`*^9, 3.680388282922524*^9}, {3.680388325476224*^9, 
   3.680388385097063*^9}, {3.680388448259755*^9, 3.680388497540736*^9}, {
   3.680388528028738*^9, 3.6803885431097383`*^9}, {3.680388576469054*^9, 
   3.680388577102873*^9}, {3.6803905201326733`*^9, 3.6803905242553577`*^9}, {
   3.6803909916740627`*^9, 3.680390991774457*^9}, {3.6803918648551626`*^9, 
   3.680391878152375*^9}, {3.680535331416498*^9, 3.680535334748164*^9}, 
   3.680535407168726*^9, {3.680621008328866*^9, 3.6806210083444924`*^9}, {
   3.680621214404108*^9, 3.6806212523943214`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwVk3s41IkexieJ7dmc1NpMni4kW65DJpfmN77fKbeRHyItJ7ZjSmldW6za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     "]]}, 
   {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwVlGs81IkexoUTtUQotlqXI6fccpszof/4/SYyRv7WIc1MSSGljBSiJLkm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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{
    FormBox["\"Temperature, C\"", TraditionalForm], 
    FormBox["\"s\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{0, 60}, {0., 0.4755064478273072}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.680388283468932*^9, 3.680388348338642*^9}, {
   3.6803884607471037`*^9, 3.6803884873846493`*^9}, {3.680388537290106*^9, 
   3.680388543983404*^9}, 3.6803885781485515`*^9, 3.680390524760724*^9, {
   3.68039093328631*^9, 3.680390947444213*^9}, 3.680391003271671*^9, {
   3.6803918700804462`*^9, 3.680391878652439*^9}, 3.680392051219309*^9, 
   3.680392124484682*^9, {3.6803922065683393`*^9, 3.6803922468319144`*^9}, {
   3.6803923711106586`*^9, 3.6803923891085577`*^9}, 3.680529295495547*^9, 
   3.6805354162011237`*^9, 3.680536472203041*^9, 3.6805369070883927`*^9, 
   3.6805369911271653`*^9, 3.680537253470351*^9, {3.6805374149828095`*^9, 
   3.680537512944117*^9}, 3.680537576764882*^9, 3.680537609142702*^9, 
   3.68053768739384*^9, {3.680537747532563*^9, 3.6805377634848948`*^9}, {
   3.6805378244285283`*^9, 3.68053783346786*^9}, 3.680537961214189*^9, 
   3.6805379920750895`*^9, 3.680541593105313*^9, 3.6806177795566854`*^9, {
   3.680621105374056*^9, 3.6806211242428665`*^9}, 3.6806212037013774`*^9, 
   3.680621253186764*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["T1 and T2 for Rb87", "Section",
 CellChangeTimes->{{3.680391535242239*^9, 3.680391559585094*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"T1", "[", 
       RowBox[{"Rb87", ",", " ", "ParaffinCell"}], "]"}], ",", " ", 
      "\[IndentingNewLine]", 
      RowBox[{"T2", "[", 
       RowBox[{"Cs133", ",", " ", "ParaffinCell"}], "]"}]}], "}"}], " ", "/.",
     " ", 
    RowBox[{"Temperature", "\[Rule]", 
     RowBox[{"t", "+", "273"}]}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", " ", "0", ",", "60"}], "}"}], ",", " ", 
   RowBox[{"Evaluated", "\[Rule]", "True"}], ",", " ", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Temperature, C\>\"", ",", " ", "\"\<s\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.680388562871792*^9, 3.680388581203521*^9}, 
   3.680390538315093*^9, {3.6803909958753743`*^9, 3.6803909959535837`*^9}, {
   3.6805353394145813`*^9, 3.680535343537267*^9}, 3.6805354047688465`*^9, {
   3.6806210083444924`*^9, 3.6806210083444924`*^9}, {3.680621261194996*^9, 
   3.6806212667118635`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwVkns4lHkbx2entiiSsOV9KxSdJClUtum+9czGDI+rRqOZWordJKITHZyi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     "]]}, 
   {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwVlGs81IkexoUTtUQotlqXI6fccpszof/4/SYyRv7WIc1MSSGljBSiJLkm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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{
    FormBox["\"Temperature, C\"", TraditionalForm], 
    FormBox["\"s\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{0, 60}, {0., 0.3873473168856456}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.680388565390605*^9, 3.6803885819691505`*^9}, 
   3.680390539500164*^9, {3.6803909333332014`*^9, 3.680390947495118*^9}, 
   3.680391003321089*^9, 3.6803920512662134`*^9, 3.6803921245360746`*^9, {
   3.6803922066152167`*^9, 3.6803922468848143`*^9}, {3.6803923711637077`*^9, 
   3.6803923891619616`*^9}, 3.6805292955535946`*^9, 3.6805354162385025`*^9, 
   3.680536472268179*^9, 3.680536907119625*^9, 3.6805369911704755`*^9, 
   3.680537253517354*^9, {3.68053741503436*^9, 3.6805375129970303`*^9}, 
   3.6805375768177643`*^9, 3.680537609205434*^9, 3.6805376874472237`*^9, {
   3.680537747587681*^9, 3.680537763547409*^9}, {3.6805378244754033`*^9, 
   3.680537833519747*^9}, 3.6805379612670135`*^9, 3.680537992122019*^9, 
   3.680541593158285*^9, 3.680617779625325*^9, {3.6806211055583515`*^9, 
   3.6806211242806463`*^9}, 3.6806212037638845`*^9, 3.6806212688588104`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Optical rotation angle per 1 fT  Cs (blue) vs Rb (yellow)  per 1cm probe \
length\
\>", "Section",
 CellChangeTimes->{{3.680391535242239*^9, 3.6803916452727776`*^9}, {
  3.680535889429841*^9, 3.680535908997297*^9}, {3.6805359592856655`*^9, 
  3.6805359799956284`*^9}, {3.68053649144514*^9, 3.6805364943931036`*^9}, {
  3.680537656849535*^9, 3.680537664136228*^9}, {3.6805379072022734`*^9, 
  3.680537912221136*^9}, {3.6805467764096613`*^9, 3.680546783210556*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"RotationPerFtRadians", "[", 
       RowBox[{"Cs133", ",", "ParaffinCell", ",", "lasers", ",", "Cs133D2"}], 
       "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"RotationPerFtRadians", "[", 
       RowBox[{"Rb87", ",", "ParaffinCell", ",", "lasers", ",", "Rb87D2"}], 
       "]"}]}], "\[IndentingNewLine]", "}"}], "   ", "/.", " ", 
    RowBox[{"Temperature", "\[Rule]", 
     RowBox[{"t", "+", "273"}]}]}], ",", " ", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"t", ",", " ", "0", ",", " ", "60"}], "}"}], ",", " ", 
   RowBox[{"Evaluated", "\[Rule]", "True"}], ",", " ", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Temperature, C\>\"", ",", " ", "\"\<rad\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6803910512177935`*^9, 3.6803911418988056`*^9}, {
   3.6803911930015917`*^9, 3.680391262432933*^9}, {3.6803912977307444`*^9, 
   3.6803913126691265`*^9}, {3.6805353500674963`*^9, 3.680535356364964*^9}, 
   3.6805354009896355`*^9, {3.680535830053074*^9, 3.680535850015561*^9}, {
   3.680536393151538*^9, 3.680536464899912*^9}, {3.680537627179716*^9, 
   3.6805376515608644`*^9}, {3.680546773223608*^9, 3.680546774239507*^9}, {
   3.6806210083444924`*^9, 3.6806210083444924`*^9}, {3.6806212861625085`*^9, 
   3.680621292842978*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwV1nk4Vd33APBCKt/kTUSKRLPhuu69ruFalq5Much0jjlDKEkvmjRJVAoZ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     "]]}, 
   {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwV1nk0VfsXAHChpCizhDJkKpJnLvbd555bXFxxDecShSgpL1QqPa+SVB4q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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{
    FormBox["\"Temperature, C\"", TraditionalForm], 
    FormBox["\"rad\"", TraditionalForm]},
  AxesOrigin->{0, 4.*^-9},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{0, 60}, {4.637527701412802*^-9, 2.2749298471156306`*^-8}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.680391099454788*^9, {3.6803911332072163`*^9, 3.6803911421956387`*^9}, {
   3.6803912573306627`*^9, 3.680391263434656*^9}, 3.680391313723345*^9, 
   3.680392051424219*^9, 3.6803921246105833`*^9, {3.6803922067151265`*^9, 
   3.680392246965352*^9}, {3.6803923713020573`*^9, 3.680392389246614*^9}, 
   3.680529295749523*^9, 3.6805354163231535`*^9, {3.680535852816548*^9, 
   3.6805358763890314`*^9}, {3.6805364222512245`*^9, 3.6805364723658743`*^9}, 
   3.6805369072523413`*^9, 3.6805369912551455`*^9, 3.680537253670393*^9, {
   3.680537415228032*^9, 3.6805375131493635`*^9}, 3.680537576967147*^9, {
   3.6805376093404694`*^9, 3.680537652328557*^9}, 3.680537687596691*^9, {
   3.680537747771923*^9, 3.680537763669507*^9}, {3.680537824597639*^9, 
   3.6805378336508927`*^9}, 3.6805379614455166`*^9, 3.680537992253532*^9, 
   3.6805415932897987`*^9, 3.6805467854955597`*^9, 3.680617779778954*^9, {
   3.6806211058065076`*^9, 3.680621124546567*^9}, 3.680621203932738*^9, 
   3.6806212943964143`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cell polarization degree  Cs (blue) vs Rb (yellow)", "Section",
 CellChangeTimes->{{3.680391706882268*^9, 3.6803917435996666`*^9}, {
  3.68039232040803*^9, 3.680392332430279*^9}, {3.68039240009788*^9, 
  3.6803924003820643`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"CellPolarizationDegree", "[", 
       RowBox[{"Cs133", ",", "ParaffinCell", ",", "lasers", ",", "Cs133D2"}], 
       "]"}], ",", " ", 
      RowBox[{"CellPolarizationDegree", "[", 
       RowBox[{"Rb87", ",", "ParaffinCell", ",", "lasers", ",", "Rb87D2"}], 
       "]"}]}], "}"}], " ", "/.", " ", 
    RowBox[{"Temperature", "\[Rule]", 
     RowBox[{"t", "+", "273"}]}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", " ", "0", ",", " ", "60"}], "}"}], ",", " ", 
   RowBox[{"Evaluated", "\[Rule]", "True"}], ",", " ", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Temperature, C\>\"", ",", " ", "\"\<fraction\>\""}], 
     "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6803898054596367`*^9, 3.6803898449332576`*^9}, 
   3.6803905440023746`*^9, {3.6803906092404604`*^9, 3.6803906878520756`*^9}, {
   3.680390728890583*^9, 3.6803907488013077`*^9}, 3.6803907802972527`*^9, {
   3.680390824752171*^9, 3.6803908377281857`*^9}, 3.6803908981740475`*^9, {
   3.6803909999029446`*^9, 3.6803909999921474`*^9}, {3.6803913631251526`*^9, 
   3.680391398241024*^9}, {3.680392334835539*^9, 3.6803923473211007`*^9}, {
   3.6805353633508253`*^9, 3.680535366287001*^9}, 3.6805353967162333`*^9, {
   3.6806210083444924`*^9, 3.680621008360118*^9}, {3.680621302359505*^9, 
   3.680621306843679*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwVkHs8lAkXx7ETXVa1kktrUeiqGeKlzDPO0ShGHllSptyGalV0QendWm12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     "]]}, 
   {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwV0ns41GkbwHGvtOglQkVvpNBBGCQVq/sezyxm1s+LUoTWod7KIRVSOqFz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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{
    FormBox["\"Temperature, C\"", TraditionalForm], 
    FormBox["\"fraction\"", TraditionalForm]},
  AxesOrigin->{0, 0.36},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{0, 60}, {0.3871451112149873, 0.997304550843963}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.680389848513755*^9, 3.680389908104759*^9, 3.680390415788274*^9, 
   3.6803905450413446`*^9, {3.6803906779997625`*^9, 3.680390688433756*^9}, 
   3.6803907360789814`*^9, 3.6803907819133615`*^9, {3.680390830741511*^9, 
   3.680390838391436*^9}, 3.68039089942765*^9, {3.680390933534057*^9, 
   3.680390947695157*^9}, 3.6803910035215235`*^9, {3.680391369485548*^9, 
   3.6803913991414757`*^9}, 3.6803920516035643`*^9, 3.6803921248010225`*^9, {
   3.6803922068996053`*^9, 3.680392247167363*^9}, {3.680392339955542*^9, 
   3.6803923893247414`*^9}, 3.680529295816592*^9, 3.6805354163846526`*^9, 
   3.680536472434597*^9, 3.6805369073195524`*^9, 3.680536991324194*^9, 
   3.6805372537711945`*^9, {3.6805374153351974`*^9, 3.680537513266471*^9}, 
   3.680537577081035*^9, 3.6805376094428787`*^9, 3.6805376877148423`*^9, {
   3.6805377478757763`*^9, 3.680537763784909*^9}, {3.6805378247130985`*^9, 
   3.680537833767689*^9}, 3.680537961545784*^9, 3.6805379923753834`*^9, 
   3.6805415933900824`*^9, 3.6806177798792124`*^9, {3.6806211060223894`*^9, 
   3.68062112476608*^9}, 3.6806212040173903`*^9, 3.6806213075972757`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Estimated noise limit in fT/sqrt[Hz] per 1cm probe length", "Section",
 CellChangeTimes->{{3.680391706882268*^9, 3.6803917435996666`*^9}, {
  3.6805364773654366`*^9, 3.680536501233881*^9}, {3.680537680690068*^9, 
  3.6805376835269923`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"TotalNoise", "[", 
       RowBox[{"Cs133", ",", "ParaffinCell", ",", "lasers", ",", "Cs133D2"}], 
       "]"}], ",", " ", 
      RowBox[{"TotalNoise", "[", 
       RowBox[{"Rb87", ",", "ParaffinCell", ",", "lasers", ",", "Rb87D2"}], 
       "]"}]}], "}"}], " ", "/.", " ", 
    RowBox[{"Temperature", "\[Rule]", 
     RowBox[{"t", "+", "273"}]}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", " ", "0", ",", " ", "60"}], "}"}], ",", " ", 
   RowBox[{"Evaluated", "\[Rule]", "True"}], ",", " ", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Temperature, C\>\"", ",", " ", "\"\<fT/sqrt(Hz)\>\""}], 
     "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6803898054596367`*^9, 3.6803898449332576`*^9}, 
   3.6803905440023746`*^9, {3.6803906092404604`*^9, 3.6803906878520756`*^9}, {
   3.680390728890583*^9, 3.6803907488013077`*^9}, 3.6803907802972527`*^9, {
   3.680390824752171*^9, 3.6803908377281857`*^9}, 3.6803908981740475`*^9, {
   3.6803909999029446`*^9, 3.6803909999921474`*^9}, {3.6803913631251526`*^9, 
   3.680391398241024*^9}, {3.6805353705092435`*^9, 3.6805353909138994`*^9}, {
   3.6805365108250675`*^9, 3.680536513106979*^9}, {3.6805375072229357`*^9, 
   3.6805375716451073`*^9}, {3.680621008360118*^9, 3.680621008360118*^9}, {
   3.6806213161468563`*^9, 3.6806213255385528`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwV13k4lF0fB3BrNPcMIqUiI0L2ZeYew+BLWshIRCmEeqhIQmlRD8qSVLYs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     "]]}, 
   {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], Opacity[
    1.], LineBox[CompressedData["
1:eJwV13k4VN8fB3CEb+qOZYhEDJUthVKyjffcGcvIKImsiVRUSgtJKqUoyU+0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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{
    FormBox["\"Temperature, C\"", TraditionalForm], 
    FormBox["\"fT/sqrt(Hz)\"", TraditionalForm]},
  AxesOrigin->{0, 3.2},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{0, 60}, {3.3553038647349487`, 9.15495479183578}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.680389848513755*^9, 3.680389908104759*^9, 3.680390415788274*^9, 
   3.6803905450413446`*^9, {3.6803906779997625`*^9, 3.680390688433756*^9}, 
   3.6803907360789814`*^9, 3.6803907819133615`*^9, {3.680390830741511*^9, 
   3.680390838391436*^9}, 3.68039089942765*^9, {3.680390933534057*^9, 
   3.680390947695157*^9}, 3.6803910035215235`*^9, {3.680391369485548*^9, 
   3.6803913991414757`*^9}, 3.6803920516035643`*^9, 3.6803921248010225`*^9, {
   3.6803922068996053`*^9, 3.680392247167363*^9}, {3.680392371579626*^9, 
   3.680392389515086*^9}, 3.680529296006218*^9, 3.680535416577168*^9, 
   3.6805364726187897`*^9, 3.6805365138323812`*^9, 3.6805369074715796`*^9, 
   3.6805369915086594`*^9, 3.6805372540340075`*^9, {3.6805374156040707`*^9, 
   3.680537513561576*^9}, 3.6805375773346615`*^9, 3.680537609705317*^9, 
   3.6805376880100007`*^9, {3.6805377481556845`*^9, 3.680537764067465*^9}, {
   3.6805378250134077`*^9, 3.680537834062559*^9}, 3.680537961886943*^9, 
   3.680537992653387*^9, 3.680541593648044*^9, 3.680617780179177*^9, {
   3.6806211063067675`*^9, 3.680621125213797*^9}, 3.680621204264352*^9, 
   3.6806213269798956`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1191, 624},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
FrontEndVersion->"10.0 for Microsoft Windows (64-bit) (September 9, 2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 176, 4, 166, "Title"],
Cell[759, 28, 1527, 34, 156, "Input"],
Cell[CellGroupData[{
Cell[2311, 66, 106, 1, 70, "Section"],
Cell[CellGroupData[{
Cell[2442, 71, 1451, 30, 72, "Input"],
Cell[3896, 103, 7006, 125, 206, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[10951, 234, 103, 1, 70, "Section"],
Cell[CellGroupData[{
Cell[11079, 239, 1050, 25, 72, "Input"],
Cell[12132, 266, 6881, 123, 206, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[19062, 395, 473, 8, 70, "Section"],
Cell[CellGroupData[{
Cell[19560, 407, 1442, 28, 132, "Input"],
Cell[21005, 437, 11993, 207, 191, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[33047, 650, 235, 3, 70, "Section"],
Cell[CellGroupData[{
Cell[33307, 657, 1445, 29, 72, "Input"],
Cell[34755, 688, 7233, 128, 213, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[42037, 822, 246, 3, 70, "Section"],
Cell[CellGroupData[{
Cell[42308, 829, 1454, 29, 72, "Input"],
Cell[43765, 860, 14458, 247, 240, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
